import * as i0 from '@angular/core';
import { EventEmitter, booleanAttribute, forwardRef, Component, ViewEncapsulation, ChangeDetectionStrategy, Input, Output, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { indentWithTab } from '@codemirror/commands';
import { indentUnit } from '@codemirror/language';
import { Annotation, Compartment, EditorState, StateEffect } from '@codemirror/state';
import { oneDark } from '@codemirror/theme-one-dark';
import { EditorView, placeholder, keymap, highlightWhitespace } from '@codemirror/view';
import { basicSetup, minimalSetup } from 'codemirror';

const External = Annotation.define();
class CodeEditor {
    /** Whether the editor is readonly. */
    get readonly() {
        return this._readonly;
    }
    set readonly(value) {
        this._readonly = value;
        this.setReadonly(value);
    }
    /** Editor's value. */
    get value() {
        return this._value;
    }
    set value(newValue) {
        this._value = newValue;
        this.setValue(newValue);
    }
    /** Editor's theme. */
    get theme() {
        return this._theme;
    }
    set theme(value) {
        this._theme = value;
        this.setTheme(value);
    }
    /** Editor's placecholder. */
    get placeholder() {
        return this._placeholder;
    }
    set placeholder(value) {
        this._placeholder = value;
        this.setPlaceholder(value);
    }
    /** Whether indent with Tab key. */
    get indentWithTab() {
        return this._indentWithTab;
    }
    set indentWithTab(value) {
        this._indentWithTab = value;
        this.setIndentWithTab(value);
    }
    /** Should be a string consisting either entirely of the same whitespace character. */
    get indentUnit() {
        return this._indentUnit;
    }
    set indentUnit(value) {
        this._indentUnit = value;
        this.setIndentUnit(value);
    }
    /** Whether the editor wraps lines. */
    get lineWrapping() {
        return this._lineWrapping;
    }
    set lineWrapping(value) {
        this._lineWrapping = value;
        this.setLineWrapping(value);
    }
    /** Whether highlight the whitespace. */
    get highlightWhitespace() {
        return this._highlightWhitespace;
    }
    set highlightWhitespace(value) {
        this._highlightWhitespace = value;
        this.setHighlightWhitespace(value);
    }
    /** Editor's language. You should set the `languages` prop at first. */
    get language() {
        return this._language;
    }
    set language(value) {
        this._language = value;
        this.setLanguage(value);
    }
    /**
     * The editor's built-in setup. The value can be set to
     * [`basic`](https://codemirror.net/docs/ref/#codemirror.basicSetup),
     * [`minimal`](https://codemirror.net/docs/ref/#codemirror.minimalSetup) or `null`.
     */
    get setup() {
        return this._setup;
    }
    set setup(value) {
        this._setup = value;
        this.setExtensions(this._getAllExtensions());
    }
    /**
     * It will be appended to the root
     * [extensions](https://codemirror.net/docs/ref/#state.EditorStateConfig.extensions).
     */
    get extensions() {
        return this._extensions;
    }
    set extensions(value) {
        this._extensions = value;
        this.setExtensions(this._getAllExtensions());
    }
    constructor(_elementRef) {
        this._elementRef = _elementRef;
        /** Whether focus on the editor after init. */
        this.autoFocus = false;
        /** Whether the editor is disabled.  */
        this.disabled = false;
        this._readonly = false;
        this._value = '';
        this._theme = 'light';
        this._placeholder = '';
        this._indentWithTab = false;
        this._indentUnit = '';
        this._lineWrapping = false;
        this._highlightWhitespace = false;
        /**
         * An array of language descriptions for known
         * [language-data](https://github.com/codemirror/language-data/blob/main/src/language-data.ts).
         */
        this.languages = [];
        this._language = '';
        this._setup = 'basic';
        this._extensions = [];
        /** Event emitted when the editor's value changes. */
        this.change = new EventEmitter();
        /** Event emitted when focus on the editor. */
        this.focus = new EventEmitter();
        /** Event emitted when the editor has lost focus. */
        this.blur = new EventEmitter();
        this._onChange = () => { };
        this._onTouched = () => { };
        this._updateListener = EditorView.updateListener.of(vu => {
            if (vu.docChanged && !vu.transactions.some(tr => tr.annotation(External))) {
                const value = vu.state.doc.toString();
                this._onChange(value);
                this.change.emit(value);
            }
        });
        // Extension compartments can be used to make a configuration dynamic.
        // https://codemirror.net/docs/ref/#state.Compartment
        this._editableConf = new Compartment();
        this._readonlyConf = new Compartment();
        this._themeConf = new Compartment();
        this._placeholderConf = new Compartment();
        this._indentWithTabConf = new Compartment();
        this._indentUnitConf = new Compartment();
        this._lineWrappingConf = new Compartment();
        this._highlightWhitespaceConf = new Compartment();
        this._languageConf = new Compartment();
    }
    _getAllExtensions() {
        return [
            this._updateListener,
            this._editableConf.of([]),
            this._readonlyConf.of([]),
            this._themeConf.of([]),
            this._placeholderConf.of([]),
            this._indentWithTabConf.of([]),
            this._indentUnitConf.of([]),
            this._lineWrappingConf.of([]),
            this._highlightWhitespaceConf.of([]),
            this._languageConf.of([]),
            this.setup === 'basic' ? basicSetup : this.setup === 'minimal' ? minimalSetup : [],
            ...this.extensions,
        ];
    }
    ngOnInit() {
        this.view = new EditorView({
            root: this.root,
            parent: this._elementRef.nativeElement,
            state: EditorState.create({ doc: this.value, extensions: this._getAllExtensions() }),
        });
        if (this.autoFocus) {
            this.view?.focus();
        }
        this.view?.contentDOM.addEventListener('focus', () => {
            this._onTouched();
            this.focus.emit();
        });
        this.view?.contentDOM.addEventListener('blur', () => {
            this._onTouched();
            this.blur.emit();
        });
        this.setEditable(!this.disabled);
        this.setReadonly(this.readonly);
        this.setTheme(this.theme);
        this.setPlaceholder(this.placeholder);
        this.setIndentWithTab(this.indentWithTab);
        this.setIndentUnit(this.indentUnit);
        this.setLineWrapping(this.lineWrapping);
        this.setHighlightWhitespace(this.highlightWhitespace);
        this.setLanguage(this.language);
    }
    ngOnDestroy() {
        this.view?.destroy();
    }
    writeValue(value) {
        if (this.view) {
            this.setValue(value);
        }
    }
    registerOnChange(fn) {
        this._onChange = fn;
    }
    registerOnTouched(fn) {
        this._onTouched = fn;
    }
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
        this.setEditable(!isDisabled);
    }
    _dispatchEffects(effects) {
        return this.view?.dispatch({ effects });
    }
    /** Sets the root extensions of the editor. */
    setExtensions(value) {
        this._dispatchEffects(StateEffect.reconfigure.of(value));
    }
    /** Sets editor's value. */
    setValue(value) {
        this.view?.dispatch({
            changes: { from: 0, to: this.view.state.doc.length, insert: value },
        });
    }
    /** Sets editor's editable state. */
    setEditable(value) {
        this._dispatchEffects(this._editableConf.reconfigure(EditorView.editable.of(value)));
    }
    /** Sets editor's readonly state. */
    setReadonly(value) {
        this._dispatchEffects(this._readonlyConf.reconfigure(EditorState.readOnly.of(value)));
    }
    /** Sets editor's theme. */
    setTheme(value) {
        this._dispatchEffects(this._themeConf.reconfigure(value === 'light' ? [] : value === 'dark' ? oneDark : value));
    }
    /** Sets editor's placeholder. */
    setPlaceholder(value) {
        this._dispatchEffects(this._placeholderConf.reconfigure(value ? placeholder(value) : []));
    }
    /** Sets editor' indentWithTab. */
    setIndentWithTab(value) {
        this._dispatchEffects(this._indentWithTabConf.reconfigure(value ? keymap.of([indentWithTab]) : []));
    }
    /** Sets editor's indentUnit. */
    setIndentUnit(value) {
        this._dispatchEffects(this._indentUnitConf.reconfigure(value ? indentUnit.of(value) : []));
    }
    /** Sets editor's lineWrapping. */
    setLineWrapping(value) {
        this._dispatchEffects(this._lineWrappingConf.reconfigure(value ? EditorView.lineWrapping : []));
    }
    /** Sets editor's highlightWhitespace. */
    setHighlightWhitespace(value) {
        this._dispatchEffects(this._highlightWhitespaceConf.reconfigure(value ? highlightWhitespace() : []));
    }
    /** Sets editor's language dynamically. */
    setLanguage(lang) {
        if (!lang) {
            return;
        }
        if (this.languages.length === 0) {
            if (this.view) {
                console.error('No supported languages. Please set the `languages` prop at first.');
            }
            return;
        }
        const langDesc = this._findLanguage(lang);
        langDesc?.load().then(lang => {
            this._dispatchEffects(this._languageConf.reconfigure([lang]));
        });
    }
    /** Find the language's extension by its name. Case insensitive. */
    _findLanguage(name) {
        for (const lang of this.languages) {
            for (const alias of [lang.name, ...lang.alias]) {
                if (name.toLowerCase() === alias.toLowerCase()) {
                    return lang;
                }
            }
        }
        console.error('Language not found:', name);
        console.info('Supported language names:', this.languages.map(lang => lang.name).join(', '));
        return null;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.3.9", ngImport: i0, type: CodeEditor, deps: [{ token: i0.ElementRef }], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "16.1.0", version: "17.3.9", type: CodeEditor, isStandalone: true, selector: "code-editor", inputs: { root: "root", autoFocus: ["autoFocus", "autoFocus", booleanAttribute], disabled: ["disabled", "disabled", booleanAttribute], readonly: ["readonly", "readonly", booleanAttribute], value: "value", theme: "theme", placeholder: "placeholder", indentWithTab: ["indentWithTab", "indentWithTab", booleanAttribute], indentUnit: "indentUnit", lineWrapping: ["lineWrapping", "lineWrapping", booleanAttribute], highlightWhitespace: ["highlightWhitespace", "highlightWhitespace", booleanAttribute], languages: "languages", language: "language", setup: "setup", extensions: "extensions" }, outputs: { change: "change", focus: "focus", blur: "blur" }, host: { classAttribute: "code-editor" }, providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => CodeEditor),
                multi: true,
            },
        ], ngImport: i0, template: ``, isInline: true, styles: [".code-editor{display:block}.code-editor .cm-editor{height:100%}\n"], changeDetection: i0.ChangeDetectionStrategy.OnPush, encapsulation: i0.ViewEncapsulation.None }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.3.9", ngImport: i0, type: CodeEditor, decorators: [{
            type: Component,
            args: [{ selector: 'code-editor', standalone: true, template: ``, host: {
                        class: 'code-editor',
                    }, encapsulation: ViewEncapsulation.None, changeDetection: ChangeDetectionStrategy.OnPush, providers: [
                        {
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(() => CodeEditor),
                            multi: true,
                        },
                    ], styles: [".code-editor{display:block}.code-editor .cm-editor{height:100%}\n"] }]
        }], ctorParameters: () => [{ type: i0.ElementRef }], propDecorators: { root: [{
                type: Input
            }], autoFocus: [{
                type: Input,
                args: [{ transform: booleanAttribute }]
            }], disabled: [{
                type: Input,
                args: [{ transform: booleanAttribute }]
            }], readonly: [{
                type: Input,
                args: [{ transform: booleanAttribute }]
            }], value: [{
                type: Input
            }], theme: [{
                type: Input
            }], placeholder: [{
                type: Input
            }], indentWithTab: [{
                type: Input,
                args: [{ transform: booleanAttribute }]
            }], indentUnit: [{
                type: Input
            }], lineWrapping: [{
                type: Input,
                args: [{ transform: booleanAttribute }]
            }], highlightWhitespace: [{
                type: Input,
                args: [{ transform: booleanAttribute }]
            }], languages: [{
                type: Input
            }], language: [{
                type: Input
            }], setup: [{
                type: Input
            }], extensions: [{
                type: Input
            }], change: [{
                type: Output
            }], focus: [{
                type: Output
            }], blur: [{
                type: Output
            }] } });

class CodeEditorModule {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.3.9", ngImport: i0, type: CodeEditorModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "17.3.9", ngImport: i0, type: CodeEditorModule, imports: [CodeEditor], exports: [CodeEditor] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "17.3.9", ngImport: i0, type: CodeEditorModule }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.3.9", ngImport: i0, type: CodeEditorModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [CodeEditor],
                    exports: [CodeEditor],
                }]
        }] });

/*
 * Public API Surface of code-editor
 */

/**
 * Generated bundle index. Do not edit.
 */

export { CodeEditor, CodeEditorModule };
//# sourceMappingURL=acrodata-code-editor.mjs.map
